@@ -1,5 +1,6 @@
 from sklearn.model_selection import train_test_split
 from sklearn.model_selection import GridSearchCV
+from sklearn.metrics import make_scorer, precision_score, recall_score, f1_score, accuracy_score, jaccard_score
 import pandas as pd
 import numpy as np
 import zipfile
@@ -8,7 +9,7 @@ import json
 import re
 
 from texta_tools.text_processor import TextProcessor
-
+from typing import List, Union
 from .pipeline import get_pipeline_builder
 from .tagging_report import TaggingReport
 from . import exceptions
@@ -45,12 +46,53 @@ class Tagger:
         self.description = description
         self.max_examples_per_class = max_examples_per_class
         # set text processor
-        self.text_processor = self._create_text_processor(embedding, mlp, custom_stop_words)    
+        self.text_processor = self._create_text_processor(embedding, mlp, custom_stop_words)
+        # set default sklearn average functions
+        self.sklearn_average_binary = "binary"
+        self.sklearn_average_multilabel = "micro"
+
+        self.scoring_function = None
+
+        # metrics map for make_scorer
+        self.metric_map = {
+            "precision": {"function": precision_score, "needs_pos_label": True},
+            "recall": {"function": recall_score, "needs_pos_label": True},
+            "f1_score": {"function": f1_score, "needs_pos_label": True},
+            "jaccard": {"function": jaccard_score, "needs_pos_label": True},
+            "accuracy": {"function": accuracy_score, "needs_pos_label": False},
+        }
 
 
     def __str__(self):
         return self.description
 
+    def _get_scorer(self, metric: str, pos_label: Union[str, int], average: str):
+        """Make custom scorer based on given metric function."""
+        custom_scorer = None
+        if metric:
+            if metric not in self.metric_map:
+                raise exceptions.InvalidScoringIdentifierError(f"Metric identifier'{metric}' is not in the list of available metrics. Available metrics: {list(self.metric_map.keys())}")
+            metric_data = self.metric_map[metric]
+            if metric_data["needs_pos_label"]:
+                custom_scorer = make_scorer(metric_data["function"], greater_is_better=True,  pos_label=pos_label, average=average)
+                self.scoring_function = f"{metric}_{average}"
+            else:
+                custom_scorer = make_scorer(metric_data["function"], greater_is_better=True)
+                self.scoring_function = metric
+        return custom_scorer
+
+
+    def _get_pos_label(self, labels: List[Union[str, int]], pos_label: Union[str, int]) -> Union[str, int]:
+        if not pos_label:
+            if len(labels) == 2:
+                if "true" in labels:
+                    pos_label = "true"
+                else:
+                    raise exceptions.PosLabelNotSpecifiedError("Label set must contain label 'true' or positive label must be specified.")
+            else:
+                # Pos label is not important on multilabel dataset
+                pos_label = None
+        return pos_label
 
     def _create_text_processor(self, embedding, mlp, custom_stop_words):
         """
@@ -86,7 +128,7 @@ class Tagger:
             raise exceptions.InvalidInputError(f"Data sample values should contain list of dicts or list of strings. Instead it is {type()}")
 
 
-    def train(self, data_sample, field_list=[]):
+    def train(self, data_sample, field_list=[], scoring=None, pos_label=None, sklearn_average=None):
         """
         Executes the pipeline and trains one vs rest classification model based on data sample.
         :param dict data_sample: Dict containing training data. Dict keys are labels and values are either lists of texts or list of documents.
@@ -102,6 +144,19 @@ class Tagger:
         # if no field list, default to all fields in the first document.
         if not field_list:
             field_list = list(list(data_sample.values())[0][0].keys())
+
+        # get labels
+        labels = list(data_sample.keys())
+        # get positive label
+        pos_label = self._get_pos_label(labels, pos_label)
+        # set average_function:
+        if not sklearn_average:
+            if len(labels) == 2:
+                sklearn_average = self.sklearn_average_binary
+            else:
+                sklearn_average = self.sklearn_average_multilabel
+        # get custom scorer:
+        scorer = self._get_scorer(scoring, pos_label, sklearn_average)
         # pipeline
         pipe_builder = get_pipeline_builder(workers=self.workers)
         pipe_builder.set_pipeline_options(self.vectorizer, self.classifier)
@@ -126,8 +181,9 @@ class Tagger:
         # dataframes
         df_train = pd.DataFrame(X_train)
         df_test = pd.DataFrame(X_test)
+
         # Use Train data to parameter selection in a Grid Search
-        gs_clf = GridSearchCV(c_pipe, c_params, n_jobs=self.workers, cv=5, verbose=False)
+        gs_clf = GridSearchCV(c_pipe, c_params, n_jobs=self.workers, cv=5, verbose=False, scoring=scorer)
         gs_clf = gs_clf.fit(df_train, y_train)
         model = gs_clf.best_estimator_
         # Use best model and test data for final evaluation
@@ -143,7 +199,8 @@ class Tagger:
             y_scores,
             num_features,
             {k:len(v) for k,v in data_sample.items()},
-            list(data_sample.keys())
+            labels,
+            pos_label
         )
         # update model & report
         self.model = model
@@ -175,7 +232,7 @@ class Tagger:
             raise exceptions.NotSupportedError("This feature is not supported with Hashing Vectorizer")
         return self.model.named_steps["union"].transformer_list[0][1].named_steps["vectorizer"].get_feature_names()
 
-    
+
     @check_model_loaded
     def get_supports(self):
         """
@@ -231,7 +288,7 @@ class Tagger:
             return self.load(tagger_path)
         except:
             raise exceptions.ModelLoadFailedError()
-    
+
 
     def load_zip(self, zip_path):
         """
